def calc_cost_from_completion(resp, pricing) -> float:
    usage = None
    for attr in ("usage", "usage_metadata"):
        usage = getattr(resp, attr, None)
        if usage:
            break
    if not usage:
        return 0.0

    prompt_keys = ("prompt_tokens", "input_tokens", "prompt_token_count")
    completion_keys = ("completion_tokens", "output_tokens", "candidates_token_count")

    pt = next((getattr(usage, k) for k in prompt_keys if hasattr(usage, k)), 0)
    ct = next((getattr(usage, k) for k in completion_keys if hasattr(usage, k)), 0)
    cost = round((pt * pricing.get("prompt", 0) + ct * pricing.get("completion", 0)), 6)
    return pt, ct, cost


import os
import sys
# 프로젝트 루트의 tracker 패키지를 찾도록 경로 추가
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import pytest
import asyncio
from tracker.cost_tracker import cost_tracker
from tracker.pricing_loader import load_pricing_yaml

# ─────────────────────────────────────────────────────────────
# 매 테스트 전후 상태 초기화
@pytest.fixture(autouse=True)
def reset_cost_tracker():
    cost_tracker.costs.clear()
    cost_tracker.token_logs.clear()
    # 설정 파일에서 실제 pricing 정보를 불러오기
    cost_tracker.pricing = load_pricing_yaml()
    yield
# ─────────────────────────────────────────────────────────────

# ─────────────────────────────────────────────────────────────
# OpenAI(ChatCompletion) 더미 스키마
class DummyCompletionUsage:
    def __init__(self, prompt_tokens: int, completion_tokens: int):
        self.prompt_tokens = prompt_tokens
        self.completion_tokens = completion_tokens

class DummyChatCompletion:
    def __init__(self, prompt_tokens: int, completion_tokens: int):
        self.usage = DummyCompletionUsage(prompt_tokens, completion_tokens)

@cost_tracker.track_cost()
def call_openai_sync(model_name: str):
    return DummyChatCompletion(prompt_tokens=15, completion_tokens=12)

@cost_tracker.track_cost()
async def call_openai_async(model_name: str):
    return DummyChatCompletion(prompt_tokens=8, completion_tokens=5)
# ─────────────────────────────────────────────────────────────


def test_openai_sync_tracks_cost_and_tokens():
    model_name = "gpt-4o-mini"
    call_openai_sync(model_name)

    provider_prices = cost_tracker.pricing.get("openai", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None, f"Price for {model_name} not found under 'openai'"

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(15 * prompt_price + 12 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [15]
    assert toks["completion_tokens"] == [12]


def test_openai_async_tracks_cost_and_tokens():
    model_name = "gpt-4o-mini"
    asyncio.run(call_openai_async(model_name))

    provider_prices = cost_tracker.pricing.get("openai", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(8 * prompt_price + 5 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [8]
    assert toks["completion_tokens"] == [5]
# ─────────────────────────────────────────────────────────────

# ─────────────────────────────────────────────────────────────
# Anthropic(Message) 더미 스키마
class DummyAnthropicUsage:
    def __init__(self, input_tokens: int, output_tokens: int):
        self.input_tokens  = input_tokens
        self.output_tokens = output_tokens

class DummyAnthropicMessage:
    def __init__(self, input_tokens: int, output_tokens: int):
        self.usage = DummyAnthropicUsage(input_tokens, output_tokens)

@cost_tracker.track_cost()
def call_anthropic_sync(model_name: str):
    return DummyAnthropicMessage(input_tokens=10, output_tokens=21)

@cost_tracker.track_cost()
async def call_anthropic_async(model_name: str):
    return DummyAnthropicMessage(input_tokens=4, output_tokens=6)
# ─────────────────────────────────────────────────────────────


def test_anthropic_sync_tracks_cost_and_tokens():
    model_name = "claude-3-5-haiku-20241022"
    call_anthropic_sync(model_name)

    provider_prices = cost_tracker.pricing.get("antrophic", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None, f"Price for {model_name} not found under 'antrophic'"

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(10 * prompt_price + 21 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [10]
    assert toks["completion_tokens"] == [21]


def test_anthropic_async_tracks_cost_and_tokens():
    model_name = "claude-3-5-haiku-20241022"
    asyncio.run(call_anthropic_async(model_name))

    provider_prices = cost_tracker.pricing.get("antrophic", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(4 * prompt_price + 6 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [4]
    assert toks["completion_tokens"] == [6]
# ─────────────────────────────────────────────────────────────

# ─────────────────────────────────────────────────────────────
# Gemini(GenerateContentResponse) 더미 스키마
class DummyGeminiUsageMetadata:
    def __init__(self, prompt_tokens: int, completion_tokens: int):
        self.prompt_token_count = prompt_tokens
        self.candidates_token_count = completion_tokens

class DummyGenerateContentResponse:
    def __init__(self, prompt_tokens: int, completion_tokens: int):
        self.usage_metadata = DummyGeminiUsageMetadata(prompt_tokens, completion_tokens)

@cost_tracker.track_cost()
def call_gemini_sync(model_name: str):
    return DummyGenerateContentResponse(prompt_tokens=4, completion_tokens=11)

@cost_tracker.track_cost()
async def call_gemini_async(model_name: str):
    return DummyGenerateContentResponse(prompt_tokens=2, completion_tokens=5)
# ─────────────────────────────────────────────────────────────


def test_gemini_sync_tracks_cost_and_tokens():
    model_name = "gemini-2.0-flash"
    call_gemini_sync(model_name)

    provider_prices = cost_tracker.pricing.get("google", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None, f"Price for {model_name} not found under 'google'"

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(4 * prompt_price + 11 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [4]
    assert toks["completion_tokens"] == [11]


def test_gemini_async_tracks_cost_and_tokens():
    model_name = "gemini-2.0-flash"
    asyncio.run(call_gemini_async(model_name))

    provider_prices = cost_tracker.pricing.get("google", {})
    model_prices = provider_prices.get(model_name)
    assert model_prices is not None, f"Price for {model_name} not found under 'google'"

    prompt_price = model_prices["prompt"]
    completion_price = model_prices["completion"]
    expected = round(2 * prompt_price + 5 * completion_price, 6)

    assert model_name in cost_tracker.costs
    assert pytest.approx(cost_tracker.costs[model_name][0], rel=1e-6) == expected

    toks = cost_tracker.token_logs[model_name]
    assert toks["prompt_tokens"]     == [2]
    assert toks["completion_tokens"] == [5]



import yaml
from pathlib import Path
from typing import Dict, Union

BASE_DIR = Path(__file__).parent
DEFAULT_PRICING_FILE = BASE_DIR / "pricing.yaml"

def load_pricing_yaml(path: Union[str, Path, None] = None) -> Dict[str, Dict[str, float]]:
    """
    Load pricing configuration from a YAML file.

    Args:
        path: Optional; path to a YAML file or directory. If None, falls back to DEFAULT_PRICING_FILE.

    Returns:
        A dict mapping provider keys to their prompt/completion pricing.
    """
    yaml_path = Path(path) if path else DEFAULT_PRICING_FILE
    # 상대 경로로 주었을 때 tracker 디렉토리의 pricing.yaml로 연결
    if not yaml_path.is_absolute():
        yaml_path = BASE_DIR / yaml_path
    with open(yaml_path, encoding="utf-8") as f:
        data = yaml.safe_load(f)
    return data


import re
from setuptools import setup, find_packages
from pathlib import Path

README = Path(__file__).parent / "README.md"

version_file = Path("tracker") / "__init__.py"
version = re.search(
    r'__version__ = ["\']([^"\']+)["\']',
    version_file.read_text(), re.M
).group(1)

setup(
    name="llm-cost-tracker",                            
    version=version,
    description="LLM API call token usage-based expense tracker",
    long_description=README.read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    author="Heondeuk Lee",
    author_email="leehd1995@gmail.com",
    url="https://github.com/hundredeuk2/TrackMyLLM",
    packages=find_packages(exclude=["tutorial", "tests*"]),
    install_requires=[
        "PyYAML>=5.1",
    ],
    python_requires=">=3.10",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    package_data={
        "tracker": ["pricing.yaml"],
    },
    license="MIT",
)



import inspect
from functools import wraps
from collections import defaultdict
from typing import Any, Callable
from .pricing_loader import load_pricing_yaml
from .utils import calc_cost_from_completion

class CostTracker:
    def __init__(self, 
                 pricing: dict[str, dict[str, float]] = None, 
                 pricing_path: str = "pricing.yaml"):
                 
        self.pricing = pricing or load_pricing_yaml(pricing_path)
        self.costs: dict[str, list[float]] = defaultdict(list)
        self.token_logs: dict[str, dict[str, list[int]]] = defaultdict(lambda: {"prompt_tokens": [], "completion_tokens": []})

    def total_cost(self, instance: Any = None) -> float:
        if instance is not None and hasattr(instance, "costs"):
            data = instance.costs.values()
        else:
            data = self.costs.values()
        return round(sum(sum(lst) for lst in data), 6)

    def track_cost(self, response_index: int = 0):
        def decorator(fn: Callable):
            is_async = inspect.iscoroutinefunction(fn)

            if is_async:
                @wraps(fn)
                async def async_wrapper(*args, **kwargs):
                    result = await fn(*args, **kwargs)
                    resp = (result[response_index]
                            if isinstance(result, (tuple, list)) else result)
                    inst = args[0] if args else None
                    
                    self.model_name = self._extract_model_name(inst, args)
                    self.check_company(self.model_name)

                    pt, ct, cost = calc_cost_from_completion(resp, self.price_detail[self.model_name])

                    # settings
                    target_costs = (inst.costs if hasattr(inst, 'costs') else self.costs)
                    target_tokens = (inst.token_logs if hasattr(inst, 'token_logs') else self.token_logs)

                    target_costs.setdefault(self.model_name, []).append(cost)
                    target_tokens.setdefault(self.model_name, {"prompt_tokens": [], "completion_tokens": []})
                    target_tokens[self.model_name]["prompt_tokens"].append(pt)
                    target_tokens[self.model_name]["completion_tokens"].append(ct)
                    return result
                
                return async_wrapper

            else:
                @wraps(fn)
                def sync_wrapper(*args, **kwargs):
                    result = fn(*args, **kwargs)
                    resp = (result[response_index]
                            if isinstance(result, (tuple, list)) else result)
                    inst = args[0] if args else None

                    self.model_name = self._extract_model_name(inst, args)
                    self.check_company(self.model_name)

                    pt, ct, cost = calc_cost_from_completion(resp, self.price_detail[self.model_name])

                    # settings
                    target_costs = (inst.costs if hasattr(inst, 'costs') else self.costs)
                    target_tokens = (inst.token_logs if hasattr(inst, 'token_logs') else self.token_logs)

                    target_costs.setdefault(self.model_name, []).append(cost)
                    target_tokens.setdefault(self.model_name, {"prompt_tokens": [], "completion_tokens": []})
                    target_tokens[self.model_name]["prompt_tokens"].append(pt)
                    target_tokens[self.model_name]["completion_tokens"].append(ct)
                    return result
                
                return sync_wrapper

        return decorator

    def _extract_model_name(self, inst, args):
        if hasattr(inst, "model_name"):
            return inst.model_name
        elif args:
            return args[0]
        else:
            return None

    def check_company(self, model_name: str):
        if model_name is None:
            raise ValueError("Model name is required for pricing lookup.")
        lower = model_name.lower()
        if "gpt" in lower or "o1" in lower or "o3" in lower or "o4" in lower:
            self.price_detail = self.pricing["openai"]
        elif "claude" in lower:
            self.price_detail = self.pricing.get("antrophic", {})
        elif "gemini" in lower:
            self.price_detail = self.pricing.get("google", {})
        else:
            raise ValueError(f"Unsupported model: {model_name}")

cost_tracker = CostTracker()